"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
require("apollo-codegen-core/lib/polyfills");
const command_1 = require("@oclif/command");
const Listr = require("listr");
const commonTasks_1 = require("../../helpers/commands/queries/commonTasks");
const engine_cli_1 = require("../../engine-cli");
const load_config_1 = require("../../load-config");
const apollo_link_1 = require("apollo-link");
const registerOperations_1 = require("../../operations/registerOperations");
const engine_1 = require("../../engine");
const assert = require("assert");
class RegisterQueries extends command_1.Command {
    async run() {
        let { flags } = this.parse(RegisterQueries);
        const defaultFlags = { addTypename: true, removeClientDirectives: true };
        flags = Object.assign({}, defaultFlags, flags);
        const tasks = new Listr([
            load_config_1.loadConfigStep(flags, false),
            {
                title: "Loading schema",
                task: ctx => {
                    if (Object.values(ctx.config.schemas).length > 1) {
                        this.error("More than one schema found.");
                    }
                    if (Object.values(ctx.config.schemas).length == 0) {
                        this.error("No schemas found.");
                    }
                    ctx.currentSchema = Object.values(ctx.config.schemas)[0];
                    if (!ctx.currentSchema.engineKey) {
                        this.error("No API key was specified. Set an Apollo Engine API key using the `--key` flag or the `ENGINE_API_KEY` environment variable.");
                    }
                }
            },
            ...commonTasks_1.getCommonTasks({ flags, errorLogger: this.error.bind(this) }),
            ...commonTasks_1.getCommonManifestTasks({ flags }),
            {
                title: "Registering operations with Apollo Engine",
                task: async (ctx, task) => {
                    task.title =
                        task.title +
                            ` service ${engine_1.getIdFromKey(ctx.currentSchema.engineKey)}`;
                    assert.notStrictEqual(typeof ctx.manifest, "undefined");
                    if (!ctx.manifest.operations || !ctx.manifest.operations.length) {
                        throw new Error("No operations were found.");
                    }
                    const variables = {
                        clientIdentity: {
                            name: flags.clientName,
                            identifier: flags.clientIdentifier || flags.clientName,
                            version: flags.clientVersion
                        },
                        serviceId: engine_1.getIdFromKey(ctx.currentSchema.engineKey),
                        operations: ctx.manifest.operations
                    };
                    await apollo_link_1.toPromise(apollo_link_1.execute(engine_1.engineLink, {
                        query: registerOperations_1.REGISTER_OPERATIONS,
                        variables,
                        context: Object.assign({ headers: { ["x-api-key"]: ctx.currentSchema.engineKey } }, (ctx.config.engineEndpoint && {
                            uri: ctx.config.engineEndpoint
                        }))
                    }))
                        .then(({ data, errors }) => {
                        if (!errors) {
                            return data.service.registerOperations;
                        }
                        throw new Error(errors.map(({ message }) => message).join("\n"));
                    })
                        .catch(e => {
                        if (e.result && e.result.errors) {
                            this.error(e.result.errors
                                .map(({ message }) => message)
                                .join("\n"));
                        }
                        else {
                            this.error(e.message);
                        }
                    });
                }
            }
        ]);
        return await tasks.run();
    }
}
RegisterQueries.description = "Register queries";
RegisterQueries.flags = Object.assign({ help: command_1.flags.help({
        char: "h",
        description: "Show command help"
    }), config: command_1.flags.string({
        description: "Path to your Apollo config file"
    }), queries: command_1.flags.string({
        description: "Path to your GraphQL queries, can include search tokens like **"
    }), addTypename: command_1.flags.boolean({
        description: "Automatically add __typename to your queries",
        allowNo: true
    }), removeClientDirectives: command_1.flags.boolean({
        allowNo: true,
        description: "Automatically remove @client and @connection directives and fields from operations"
    }), clientIdentifier: command_1.flags.string({
        description: "Identifier for the client which will match ids from client traces, will use clientName if not provided"
    }), clientName: command_1.flags.string({
        required: true,
        description: "Name of the client that the queries will be attached to"
    }), clientVersion: command_1.flags.string({
        description: "The version of the client that the queries will be attached to"
    }) }, engine_cli_1.engineFlags, { tagName: command_1.flags.string({
        description: "Name of the template literal tag used to identify template literals containing GraphQL queries in Javascript/Typescript code",
        default: "gql"
    }) });
exports.default = RegisterQueries;
//# sourceMappingURL=register.js.map