"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const crypto_1 = require("crypto");
const assert = require("assert");
const loading_1 = require("apollo-codegen-core/lib/loading");
const graphql_1 = require("apollo-codegen-core/lib/utilities/graphql");
const apollo_engine_reporting_1 = require("apollo-engine-reporting");
const config_1 = require("../../../config");
const taskResolveDocumentSets = () => ({
    title: "Resolving GraphQL document sets",
    task: async (ctx) => {
        assert.notStrictEqual(typeof ctx.config, "undefined");
        assert.strictEqual(typeof ctx.documentSets, "undefined");
        ctx.documentSets = await config_1.resolveDocumentSets(ctx.config, false);
    }
});
const taskScanForOperations = ({ flags }) => ({
    title: "Scanning for GraphQL queries",
    task: (ctx, task) => {
        assert.strictEqual(typeof ctx.queryDocuments, "undefined");
        ctx.queryDocuments = loading_1.loadQueryDocuments(ctx.documentSets[0].documentPaths, flags.tagName);
        task.title = `Scanning for GraphQL queries (${ctx.queryDocuments.length} found)`;
    }
});
const taskIsolateOperationsAndFragments = ({ errorLogger }) => ({
    title: "Isolating operations and fragments",
    task: ctx => {
        assert.strictEqual(typeof ctx.fragments, "undefined");
        assert.strictEqual(typeof ctx.operations, "undefined");
        const { fragments, operations } = loading_1.extractOperationsAndFragments(ctx.queryDocuments, errorLogger);
        ctx.fragments = fragments;
        ctx.operations = operations;
    }
});
const taskCombineOperationsAndFragments = ({ errorLogger }) => ({
    title: "Combining operations and fragments",
    task: ctx => {
        assert.strictEqual(typeof ctx.fullOperations, "undefined");
        ctx.fullOperations = loading_1.combineOperationsAndFragments(ctx.operations, ctx.fragments, errorLogger);
    }
});
function getCommonTasks({ flags, errorLogger }) {
    return [
        taskResolveDocumentSets(),
        taskScanForOperations({ flags }),
        taskIsolateOperationsAndFragments({ errorLogger }),
        taskCombineOperationsAndFragments({ errorLogger })
    ];
}
exports.getCommonTasks = getCommonTasks;
const manifestOperationHash = (str) => crypto_1.createHash("sha256")
    .update(str)
    .digest("hex");
const engineSignature = (_TODO_operationAST) => {
    return apollo_engine_reporting_1.defaultSignature(_TODO_operationAST, "TODO");
};
const manifestRecordForOperation = (operationAST, settings = {
    addTypename: true,
    removeClientDirectives: true
}) => {
    let document = operationAST;
    if (settings.addTypename) {
        document = graphql_1.withTypenameFieldAddedWhereNeeded(document);
    }
    if (settings.removeClientDirectives) {
        document = graphql_1.removeClientDirectives(graphql_1.removeConnectionDirectives(document));
    }
    const printed = apollo_engine_reporting_1.printWithReducedWhitespace(apollo_engine_reporting_1.sortAST(apollo_engine_reporting_1.hideLiterals(document))).trim();
    if (!printed)
        return;
    return {
        signature: manifestOperationHash(printed),
        document: printed,
        metadata: {
            engineSignature: engineSignature(document)
        }
    };
};
const taskGenerateManifest = ({ flags }) => ({
    title: "Generating manifest",
    task: ctx => {
        assert.strictEqual(Array.isArray(ctx.fullOperations), true);
        assert.strictEqual(typeof ctx.manifest, "undefined");
        const operations = ctx.fullOperations
            .map(operationAST => manifestRecordForOperation(operationAST, {
            addTypename: flags.addTypename,
            removeClientDirectives: flags.removeClientDirectives
        }))
            .filter(Boolean);
        ctx.manifest = {
            version: 1,
            operations
        };
    }
});
function getCommonManifestTasks({ flags }) {
    return [taskGenerateManifest({ flags })];
}
exports.getCommonManifestTasks = getCommonManifestTasks;
//# sourceMappingURL=commonTasks.js.map